; HP IDE Interface Code 12/6/07
; This source is compatible with the AS31 assembler
; Configured for "Paul" board Rev5 and edited by Terry Newton
; HP interface code was written (and recovered) by Bob Shannon
; The IDE code was written by Paul Stoffregen, the latest
; version and an overview of the interface design is at:
; http://www.pjrc.com/tech/8051/ide/index.html
;
; Note - preliminary info, may contain errors.
; Essentially it's been confirmed to work, although there
; still may be an issue with the status output after doing
; a block write to disk, I had to hack HP-IPL/OS' <IDE word
; to always return 0 to get it to work properly.
;
; Drive connection details...
;
; PB7 O--------------------------------------------.
; PB6 O------------------------------------------. |
; PB5 O----------------------------------------. | |
; PB4 O--------------------------------------. | | |
; PB3 O------------------------------------. | | | |
; PB2 O----------------------------------. | | | | |
; PB1 O--------------------------------. | | | | | |
; PB0 O------------------------------. | | | | | | |
;                           1        | | | | | | | |
;         .--------|>o-------O O---. | | | | | | | |
; PA7 O---|------------------O O---|-' | | | | | | |
; PA6 O---|------------------O O---|---' | | | | | |
; PA5 O---|------------------O O---|-----' | | | | |
; PA4 O---|------------------O O---|-------' | | | |
; PA3 O---|------------------O O---|---------' | | |
; PA2 O---|------------------O O---|-----------' | |
; PA1 O---|------------------O O---|-------------' |
; PA0 O---|------------------O O---|---------------'
;         |              .---O x   |
;         |             _|_  O O---*  IDE Header
; PC7 O---'   .----|>o-------O O---*  Looking at top
; PC6 O-------|-.       .----O O---*  of dev board
; PC5 O-------' `--|>o--'    O O   |  x=remove pin
; PC4 O---------.            O O---*---.
; PC3 O-------. |            O O   |  _|_
; PC2 O-----. | |      .-----O O   |
; PC1 O-----|-|-|------' .---O O---|---.
; PC0 O---. | `-|--|>o---|---O O---|-. |
;         `-|---|--------' .-O O---* | |
;           |   `--|>o-----|-------|-' |
;           `--------------|-------|---'
; +5V O---*--------470-.   `---.   |
;        _|_           |  ^^   |   |  Inverters
;    .1u -.-           `--|>|--'   |  74HC04 etc
;         |             Activity   |
; GND O---*------------------------'
;
; HP Interface details...
;
; 2nd 8255 on the Paul board are labeled PD, PE and PF (for A,B,C).
; PD - I/O data bits 0-7
; PE - I/O data bits 8-15
; PF bit 0 - input from mode switch, high for normal, low to debug
; PF bit 1 - input from HP command line, high to trigger action
; 8052 port 1 is used to output control signals...
; bit 0 - high when executing command
; bit 1 - high when operating
; bit 2 - high when block input from HP being performed (disk write)
; bit 3 - high when block output to HP being performed (disk read)
; bit 4 - high if an error occured
; bit 5 - low to high to latch output data into register to send to HP
; bit 6 - high when reading data from HP, low when writing to HP (d_data)
; bit 7 - low to high to indicate completion, connected to HP flag input
;
; NOTE!!! P1 has very feeble drive when high (~20uA, data sheet says as low
; as 1uA, ouch), bits 5-7 should be pulled high with external 10K or so R's
; and for a 12V-compatible flag an extra emitter-follower stage may be needed.
; I wired LED's + to LED to 2.2K to P1.0 to P1.5, it looks like my LED's will
; be inverted in meaning... should have wired them directly from the P1 out
; bits to ground with 2.2K pullups to +5V, then a "0" would extinguish.
;
; To buffer the data coming from the HP a pair of 74'244 or 74'245 chips with
; the output enable(s) coming from an inverted P1.6 (d_data), the leftover
; inverter from the IDE interface will do. If using a '245 then the direction
; pin needs to be high or low to make it go in the proper direction (tbd by
; the layout, set so the input is coming from HP). Outputs of the '24x's
; are wired to the inputs of a pair of 74'374 chips for registers, and to the
; I/O data bits 0-15 with 470 ohm resistors in between to prevent contention
; damage if mis-programmed. The LED's on the Paul Rev5 board must be removed.
; Port bit P1.5 connects to the clock input of the '374's, the outputs of
; the '374's are always enabled.
;
; That pretty much describes the interface if connecting to a TTL interface
; such as a microcircuit board, the inputs (from the HP outs) should be
; pulled low with hi-val resistors (10K or so) esp if using CMOS parts.
; Some series resistance on every data, command and flag lines is probably
; a good idea. [perhaps 100 ohms, enough to limit current if miswired]
;
; A ground-true interface such as the "+16 BIT DUP REG" board I'm using for
; testing requires more circuitry. This board pulls inputs up to 8 volts with
; a ~800 ohm resistor for "0", grounded inputs read as "1" (~10ma required).
; Outputs are pulled up to 12 volts with a 10K resistor to output "0", and
; grounded by the interface to output a "1" with about 12ma sink capacity.
; The pinout of the interface looking at the back of the card inserted
; into the HP mini is...
;                                                                 (cable side)
;   ---- HP inputs, from IDE interface outputs ----                 flag   gnd
;   0  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15                   |  .----
;   O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O
;   O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O  O gnd
;   0  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15                   |  `---- 
;   ---- HP outputs, to IDE interface inputs ------                command
;
; Each IDE interface input consists of a 220 ohm resistor from the HP out to
; the junction of a 8.2K resistor to ground, a switching diode to +5V, and the
; input of a 74'14 inverting trigger. This circuit is also used to buffer and
; invert the HP command output. For mine I used 2.2K and 10K.
;
; Each IDE interface output consists of a 1K resistor from the '374 output pin
; to the base of an NPN transistor, emitter grounded. Collector is connected
; to the HP input via a 10 ohm series resistor. This circuit is also used to
; buffer and invert the HP flag input. The flag out from the 8052 needs an
; emitter follower to ensure adequate current drive, with the 10K pin pullup
; I used 2.2K to the base of an NPN transistor, collector to +, emitter to
; a 1K to ground, 82 ohms from emitter to raw flag out to avoid damage if the
; line is shorted then onto the level shifter output circuit. [I don't know
; yet if this is TTL-board compatible, might need to lower the 1K emitter
; load to 470 ohms, or perhaps eliminate the follower and use a 3.3K pullup]
;
; For the "+16 BIT DUP REG" board the jumpers need to be set as follows...
; W4 to B, W5 to B (otherwise scrambled data! total mess if set to A...)
; W1 to +12, W2 to -12, W3 to -2, W6 to B, W7 to A
; W9-W12 out, W13 to A, W14 to A [I couldn't find these on the board I have
;  resulting in a failure returning drive status, to work around the <IDE
;  word was hacked to always push 0: "<IDE" $DEFADR INC 2400 PUT ]
;  
;-------------------------------------------------------------
; THIS IS RECOVERED CODE
; Attempting to debug the TDOS problem where
; seeking to DRV 1 and reading 1K causes a disk
; error state, apparently a problem translating
; the 1K block address into an ATA LBA value.

; FIXED BUGS in blk_2_lba code, removing debug
; statements for speed, resetting address for
; FLASH rom storage.

; 8/23/05, attempting fix to hpdiskb, which would not
; boot correctly, but probably calculates the LBA's
; just fine.  When a bootrom load was attempted it
; would get stuck in a loop.  The problem seems to
; be the clocking of the flag FF by up_stat.

; TESTED 8/25/05, works with DGEN / DBOOT, need
; to test boot rom, but I'm assuming that if DBOOT
; works, so will the IDE loader rom.

; HP1000 ATA disk controller version 1.0 canidate
; This version supports RDB and WRB functions with
; a transfer lenght of zero for disk to disk block
; copy operations.

; All debugging statements commented out.

;.equ	location, 0x2100
.equ	location, 0x8000	; store in flash rom
		
;load into NVRAM for debugging

.equ	buff_1, 0x3000		;first 512 bytes of block buffer
.equ	buff_2, 0x3200		;second 512 bytes of block buffer
.equ	buff_3, 0x3400		;third 512 bytes of block buffer
.equ	buff_4, 0x3600		;fourth 512 bytes of block buffer
.equ	buffer, 0x3800		;a 512 byte buffer2:

;------------------------------------------------------------------

; Hardware Configuration

;added equates for 2nd 8255 for HP interface...
;.equ	hp_porta, 0x6000
.equ	hp_porta, 0xF900
;.equ	hp_portb, 0x6001
.equ	hp_portb, 0xF901
;.equ	hp_portc, 0x6002
.equ	hp_portc, 0xF902
;.equ	hp_config, 0x6003
.equ	hp_config, 0xF903

;8255 chip.  Change these to specify where the 8255 is addressed,
;and which of the 8255's ports are connected to which ide signals.
;The first three control which 8255 ports have the control signals,
;upper and lower data bytes.  The last two are mode setting for the
;8255 to configure its ports, which must correspond to the way that
;the first three lines define which ports are connected.

;.equ	ide_8255_lsb, 0x4000
.equ	ide_8255_lsb, 0xF800
	
;lower 8 bits
;.equ	ide_8255_msb, 0x4001
.equ	ide_8255_msb, 0xF801
	
;upper 8 bits
;.equ	ide_8255_ctl, 0x4002
.equ	ide_8255_ctl, 0xF802
	
;control lines
;.equ	cfg_8255, 0x4003
.equ	cfg_8255, 0xF803
.equ	rd_ide_8255, 10010010b
	
;ide_8255_ctl out, ide_8255_lsb/msb input
.equ	wr_ide_8255, 10000000b		;all three ports output

;ide control lines for use with ide_8255_ctl.  Change these 8
;constants to reflect where each signal of the 8255 each of the
;ide control signals is connected.  All the control signals must
;be on the same port, but these 8 lines let you connect them to
;whichever pins on that port.

.equ	ide_a0_line, 0x01	
;direct from 8255 to ide interface

.equ	ide_a1_line, 0x02	
;direct from 8255 to ide interface

.equ	ide_a2_line, 0x04	
;direct from 8255 to ide interface

.equ	ide_cs0_line, 0x08	
;inverter between 8255 and ide interface

.equ	ide_cs1_line, 0x10	
;inverter between 8255 and ide interface

.equ	ide_wr_line, 0x20	
;inverter between 8255 and ide interface

.equ	ide_rd_line, 0x40	
;inverter between 8255 and ide interface

.equ	ide_rst_line, 0x80	
;inverter between 8255 and ide interface

;------------------------------------------------------------------
; More symbolic constants... these should not be changed, unless of
; course the IDE drive interface changes, perhaps when drives get
; to 128G and the PC industry will do yet another kludge.
;some symbolic constants for the ide registers, which makes the
;code more readable than always specifying the address pins

.equ	ide_data,	ide_cs0_line
.equ	ide_err,	ide_cs0_line + ide_a0_line
.equ	ide_sec_cnt,	ide_cs0_line + ide_a1_line
.equ	ide_sector,	ide_cs0_line + ide_a1_line + ide_a0_line
.equ	ide_cyl_lsb,	ide_cs0_line + ide_a2_line
.equ	ide_cyl_msb,	ide_cs0_line + ide_a2_line + ide_a0_line
.equ	ide_head,	ide_cs0_line + ide_a2_line + ide_a1_line
.equ	ide_command,	ide_cs0_line + ide_a2_line + ide_a1_line + ide_a0_line
.equ	ide_status,	ide_cs0_line + ide_a2_line + ide_a1_line + ide_a0_line
.equ	ide_control,	ide_cs1_line + ide_a2_line + ide_a1_line
.equ	ide_astatus,	ide_cs1_line + ide_a2_line + ide_a1_line + ide_a0_line

;IDE Command Constants.  These should never change.

.equ	ide_cmd_recal, 0x10
.equ	ide_cmd_read, 0x20
.equ	ide_cmd_write, 0x30
.equ	ide_cmd_init, 0x91
.equ	ide_cmd_id, 0xEC
.equ	ide_cmd_spindown, 0xE0
.equ	ide_cmd_spinup, 0xE1
;------------------------------------------------------------------
;internal ram usage

.equ	lba, 0x10			;4 bytes, 28 bit ATA Logical Block Address
.equ	blk, 0x14			;4 bytes, 26 bit 1K word block address
.equ	tmp, 0x18			;temporary value
.equ	p1_buf, 0x20			;port1 state variable
.equ	dc_cmd, 0x21			;disk controller command byte
.equ	dc_stat, 0x22			;disk controller status byte
.equ	dc_para, 0x23			;disk controller parameter byte
.equ	dc_len, 0x24			;block transfer lenght counter
.equ	ide_error, 0x25			;ide drive error code byte
.equ	hp_lsb, 0x26			;HP interface data buffer lsb
.equ	hp_msb, 0x27			;HP interface data buffer msb
.equ	stack, 0x40

;routines within paulmon2.  To make this code stand-alone, just copy
;and paste these bits of code from the paulmon2.asm file.

.equ	cout, 0x0030           	 	;Send Acc to serial port
.equ	cin, 0x0032             	;Get Acc from serial port
.equ	phex, 0x0034            	;Print Hex value of Acc
.equ	phex16, 0x0036			;print hex value of DPTR
.equ	pstr, 0x0038            	;Print string pointed to by DPTR,
.equ	ghex16, 0x003c			;get 16-bit hex value into DPTR
.equ	upper, 0x0040           	;Convert Acc to uppercase
.equ	newline, 0x0048         	;print CR/LF (13 and 10)
.equ	pint8u, 0x004D          	;print Acc at an integer, 0 to 255
.equ	pint16u, 0x0053         	;print DPTR as an integer, 0 to 65535
.equ 	cin_filter, 0x0062      	;get a character, but look for esc sequences

;------------------------------------------------------------------

; Main Program, a simple menu driven interface.

.org	location
.db     0xA5,0xE5,0xE0,0xA5		;signiture bytes
;.db     253,255,0,0			;id, 253=startup 35=don't start
.db     35,255,0,0
; change ID string to 253,255,0,0 for auto-start, but use
; the 35,255,0,0 version for debugging.
.db     0,0,0,0
.db     0,0,0,0
.db     0,0,0,0
.db     0,0,0,0
.db     0,0,0,0				;user defined
.db     255,255,255,255			;length and checksum (255=unused)
.db     "HP1000 Disk Controller",0
.org    location+64             	;executable code begins here

begin:
	mov	sp, #stack
	mov	dptr, #msg_1		;print a welcome message
	lcall	pstr

;initialize the drive.  If there is no drive, this may hang

	acall	ide_init

;get the drive id info.  If there is no drive, this may hang

	acall	drive_id		;save disk parameters in lba buffer!

;run the disk controller startup code

	ljmp	dc_start		;long jump to disk controller startup code

;-----------------------------------------------------------------------

;command line debug mode code begins

debug:

;get the drive id info.  If there is no drive, this may hang
;	acall	drive_id
; print the drive's model number

	mov	dptr, #msg_mdl
	lcall	pstr
	mov	dptr, #buffer + 60
	mov	r0, #40
	acall	print_name
	lcall	newline

; print the drive's serial number

	mov	dptr, #msg_sn
	lcall	pstr

	mov	dptr, #buffer + 20
	mov	r0, #20
	acall	print_name
	lcall	newline

; print the drive's cylinder, head, and sector specs

	mov	dptr, #msg_cy
	lcall	pstr
	mov	dptr, #buffer + 2
	acall	print_parm
	mov	dptr, #msg_hd
	lcall	pstr
	mov	dptr, #buffer + 6
	acall	print_parm
	mov	dptr, #msg_sc
	lcall	pstr
	mov	dptr, #buffer + 12
	acall	print_parm
	lcall	newline
	lcall	newline

; default position will be first block (master boot record)

	clr	a
	mov	lba+0, a
	mov	lba+1, a
	mov	lba+2, a
	mov	lba+3, a

main_loop:				;print a 1-line prompt

	mov	dptr, #msg_l
	lcall	pstr
	mov	a, lba+3
	lcall	phex
	mov	a, lba+2
	lcall	phex
	mov	a, lba+1
	lcall	phex
	mov	a, lba+0
	lcall	phex
	mov	dptr, #msg_pmt
	lcall	pstr
	lcall	cin
	lcall	upper

main1:

	cjne	a, #'R', main2		;read a sector
	mov	dptr, #buffer
	acall	read_sector
	jz	main1b
	push	acc
	mov	dptr, #msg_err
	lcall	pstr
	pop	acc
	lcall	phex
	lcall	newline
	ajmp	main_loop

main1b:

	mov	dptr, #msg_rd
	lcall	pstr
	ajmp	main_loop

main2:

	cjne	a, #'W', main3		;write a sector
	mov	dptr, #msg_sure
	lcall	pstr
	lcall	cin
	lcall	upper
	cjne	a, #'Y', main2c
	mov	dptr, #buffer
 	acall	write_sector
	jz	main2b
	push	acc
	mov	dptr, #msg_err
	lcall	pstr
	pop	acc
	lcall	phex
	lcall	newline
	ajmp	main_loop

main2b:

	mov	dptr, #msg_wr
	lcall	pstr

main2c:

	ajmp	main_loop

main3:

	cjne	a, #'L', main4		;set the logical block address
	mov	dptr, #msg_cyh
	lcall	pstr
	lcall	ghex32_lba
	jc	main3b
	mov	lba+0, r3
	mov	lba+1, r4
	mov	lba+2, r5
	mov	lba+3, r6

main3b:

	lcall	newline
	ajmp	main_loop

main4:

	cjne	a, #'U', main5		;cause the drive to spin up
	acall	spinup
	ajmp	main_loop

main5:

	cjne	a, #'D', main6		;cause the drive to spin down
	acall	spindown
	ajmp	main_loop

main6:

	cjne	a, #'Q', main7		;quit
	ljmp	0

main7:

	cjne	a, #'H', main8
	acall	hexdump
	ajmp	main_loop

main8:

	cjne	a, #'I', main9		;interface tests
	ljmp	int_tests

main9:
	ajmp	main_loop

msg_1:		.db	"HP1000 ATA Disk Controller Program",13,10,13,10,0
msg_mdl:	.db	"Model: ",0
msg_sn:		.db	"S/N:   ",0
msg_cy:		.db	"Cylinders: ", 0
msg_hd:		.db	", Heads: ", 0
msg_sc:		.db	", Sectors: ", 0
msg_cyh:	.db	"Enter LBA (in hex): ", 0
msg_l:		.db	"LBA=0x",0
msg_pmt:	.db	", (R)ead (W)rite (L)BA (U)p (D)own (H)exdump (I)nterface (Q)uit",13,10,0
msg_sure:	.db	"Warning: this will change data on the drive, are you sure?",13,10,0
msg_rd:		.db	"Sector Read OK",13,10,0
msg_wr:		.db	"Sector Write OK",13,10,0
msg_err:	.db	"Error, code = ",0

;------------------------------------------------------------------

; Routines that talk with the IDE drive, these should be called by
; the main program.
	;read a sector, specified by the 4 bytes in "lba",	
;Return, acc is zero on success, non-zero for an error

read_sector:

	acall	ide_busy		;make sure drive is ready
	acall	wr_lba			;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_read
	acall	ide_wr			;ask the drive to read it
	acall	ide_drq			;wait until it's got the data
	jb	acc.0, get_err
	mov	dptr, #buffer		;point to 512 byte buffer
	acall	read_data		;grab the data
	clr	a
	ret

;when an error occurs, we get acc.0 set from a call to ide_drq
;or ide_busy (which read the drive's status register).  If
;that error bit is set, we should jump here to read the drive's	
;explaination of the error, to be returned to the user.  If for	
;some reason the error code is zero (shouldn't happen), we'll	
;return 255, so that the main program can always depend on a	
;return of zero to indicate success.

get_err:

	mov	a, #ide_err
	acall	ide_rd
	mov	a, r2
	jz	gerr2a
	ret

gerr2a:

	mov	a, #255
	ret
	
;write a sector, specified by the 4 bytes in "lba",	
;whatever is in the buffer gets written to the drive!	
;Return, acc is zero on success, non-zero for an error

write_sector:

	acall	ide_busy		;make sure drive is ready
	acall	wr_lba			;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_write
	acall	ide_wr			;tell drive to write a sector
	acall	ide_drq			;wait unit it wants the data
	jb	acc.0, get_err
	mov	dptr, #buffer		;point to lba buffer
	acall	write_data		;give the data to the drive
	acall	ide_busy		;wait until the write is complete
	jb	acc.0, get_err
	clr	a
	ret

;do the identify drive command, and return with the buffer
;filled with info about the drive

drive_id:
	acall	ide_busy
	mov	a, #ide_command
	mov	r2, #ide_cmd_id
	acall	ide_wr
	acall	ide_drq
	mov	dptr, #buffer
	acall	read_data
	ret

;tell the drive to spin up

spinup:

	mov	r2, #ide_cmd_spinup

spup2:
	mov	a, #ide_command
	acall	ide_wr
	acall	ide_busy
	ret

;tell the drive to spin down

spindown:
	acall	ide_busy
	mov	r2, #ide_cmd_spindown
	sjmp	spup2

;initialize the ide drive

ide_init:
	mov	a, #ide_head
	mov	r2, #10100000b
	acall	ide_wr			;select the master device
	mov	a, #ide_status
	acall	ide_rd
	mov	a, r2

;should probably check for a timeout here

	jnb	acc.6, ide_init		;wait for RDY bit to be set
	jb	acc.7, ide_init		;wait for BSY bit to be clear
	mov	a, #ide_head
	mov	r2, #0x20
	acall	ide_wr			;what should this config parm be?
	mov	a, #ide_sec_cnt
	mov	r2, #64
	acall	ide_wr			;what should this config parm be?
	mov	a, #ide_command
	mov	r2, #ide_cmd_init
	acall	ide_wr			;do init parameters command
	acall	ide_busy
	mov	a, #ide_command
	mov	r2, #ide_cmd_recal	;do recal command (is this necessary?)
	acall	ide_wr
	acall	ide_busy
	ret

;------------------------------------------------------------------
; Not quite as low, low level I/O.  These routines talk to the drive,
; using the low level I/O.  Normally a main program should not call
; directly to these.

;Read a block of 512 bytes (one sector) from the drive
;and store it in memory @ DPTR

read_data:

	mov	r5, #0

rdblk2:

	push	dph
	push	dpl
	mov	a, #ide_data
	acall	ide_rd
	pop	dpl
	pop	dph
	mov	a, r2
	movx	@dptr, a
	inc	dptr
	mov	a, r3
	movx	@dptr, a
	inc	dptr
	djnz	r5, rdblk2
	ret

;Write a block of 512 bytes (at DPTR) to the drive

write_data:

	mov	r5, #0

wrblk2:

	movx	a, @dptr
	mov	r2, a
	inc	dptr
	movx	a, @dptr
	mov	r3, a
	inc	dptr
	push	dph
	push	dpl
	mov	a, #ide_data
	acall	ide_wr
	pop	dpl
	pop	dph
	djnz	r5, wrblk2
	ret

;write the logical block address to the drive's registers

wr_lba:

	mov	a, lba+3
	anl	a, #0x0F
	orl	a, #0xE0
	mov	r2, a
	mov	a, #ide_head
	acall	ide_wr
	mov	a, #ide_cyl_msb
	mov	r2, lba+2
	acall	ide_wr
	mov	a, #ide_cyl_lsb
	mov	r2, lba+1
	acall	ide_wr
	mov	a, #ide_sector
	mov	r2, lba+0
	acall	ide_wr
	mov	a, #ide_sec_cnt
	mov	r2, #1
	acall	ide_wr
	ret

;Wait for the ide drive to not be busy.
;Returns the drive's status in Acc

ide_busy:
	mov	a, #ide_status		;wait for RDY bit to be set
	acall	ide_rd
	mov	a, r2

;should probably check for a timeout here

	jb	acc.7, ide_busy
	ret

;Wait for the drive to be ready to transfer data.
;Returns the drive's status in Acc
ide_drq:
	mov	a, #ide_status		;wait for DRQ bit to be set
	acall	ide_rd
	mov	a, r2


;should probably check for a timeout here

	jb	acc.7, ide_drq		;wait for BSY to be clear
	jnb	acc.3, ide_drq		;wait for DRQ to be set
	ret

;------------------------------------------------------------------
; Low Level I/O to the drive.  These are the routines that talk
; directly to the drive, via the 8255 chip.  Normally a main
; program would not call to these.

	;Do a read bus cycle to the drive, using the 8255.  This
	;is slow, because we have to manipulate the 8255 and use
	;the 8051's limited moxv and movx to do it (via dptr).
	;Note that the drive is read using MOVC, to run on a board
	;where the 8255 is read using PSEN.  If your board uses
	;RD instead of PSEN, chance the MOVC's to MOVX's.
	;input acc = ide regsiter address
	;output r2 = lower byte read from ide drive
	;output r3 = upper byte read from ide drive
	;dptr is changed
ide_rd:
	push	acc
	mov	dptr, #cfg_8255
	mov	a, #rd_ide_8255
	movx	@dptr, a		;config 8255 chip, read mode
	mov	dptr, #ide_8255_ctl
	pop	acc
	movx	@dptr, a		;drive address onto control lines
	orl	a, #ide_rd_line	
	movx	@dptr, a		;assert read pin
	mov	dptr, #ide_8255_msb
	movx	a, @dptr		;read the upper byte
	mov	r3, a
	mov	dptr, #ide_8255_lsb
	movx	a, @dptr		;read the lower byte
	mov	r2, a
	mov	dptr, #ide_8255_ctl
	clr	a
	movx	@dptr, a		;deassert all control pins
	ret

	;Do a write bus cycle to the drive, via the 8255
	;input acc = ide register address
	;input r2 = lsb to write
	;input r3 = msb to write
	;dptr is changed

ide_wr:

	push	acc
	mov	dptr, #cfg_8255
	mov	a, #wr_ide_8255
	movx	@dptr, a		;config 8255 chip, write mode
	mov	dptr, #ide_8255_lsb
	mov	a, r2
	movx	@dptr, a		;drive lower lines with lsb (r2)
	mov	dptr, #ide_8255_msb
	mov	a, r3
	movx	@dptr, a		;drive upper lines with msb (r3)
	mov	dptr, #ide_8255_ctl
	pop	acc
	movx	@dptr, a		;drive address onto control lines
	orl	a, #ide_wr_line	
	movx	@dptr, a		;assert write pin
	nop
	clr	a
	movx	@dptr, a		;deassert all control pins
	mov	dptr, #cfg_8255
	mov	a, #rd_ide_8255
	movx	@dptr, a		;config 8255 chip, read mode
	ret

	;do a hard reset on the drive, by pulsing its reset pin.
	;this should usually be followed with a call to "ide_init".

ide_hard_reset:

	mov	dptr, #cfg_8255
	mov	a, #wr_ide_8255
	movx	@dptr, a		;config 8255 chip, write mode
	mov	dptr, #ide_8255_ctl
	mov	a, #ide_rst_line
	movx	@dptr, a		;hard reset the disk drive
	mov	r2, #250
	djnz	r2, *			;delay > 25 us (reset pulse width)
	clr	a
	movx	@dptr, a		;no ide control lines asserted
	ret

;------------------------------------------------------------------
; Some additional serial I/O routines not available in PAULMON

	;print a 16 bit number, located at DPTR

print_parm:

	movx	a, @dptr
	push	acc
	inc	dptr
	movx	a, @dptr
	mov	dph, a
	pop	dpl
	ljmp	pint16u

;print a string, no more than R0 chars long

print_name:

	movx	a, @dptr
	jz	pn_end
	lcall	cout
	inc	dptr
	djnz	r0, print_name

pn_end:	ret

;get a 32 bit input, in hex

ghex32_lba:

	mov	r2, #8
	mov	r3, #0
	mov	r4, #0
	mov	r5, #0
	mov	r6, #0

gh32c:

	lcall	cin_filter
	lcall	upper
	cjne	a, #27, gh32d
	setb	c
	ret

gh32d:

	cjne	a, #8, gh32f
	sjmp	gh32k

gh32f:

	cjne	a, #127, gh32g

gh32k:
	cjne	r2, #8, gh32e
	sjmp	gh32c

gh32e:

	lcall	cout
	mov	r0, #4

gh32yy:

	clr	c
	mov	a, r6
	rrc	a
	mov	r6, a
	mov	a, r5
	rrc	a
	mov	r5, a
	mov	a, r4
	rrc	a
	mov	r4, a
	mov	a, r3
	rrc	a
	mov	r3, a
	djnz	r0, gh32yy
	inc	r2
	sjmp	gh32c

gh32g:

	cjne	a, #13, gh32i
	clr	c
	ret

gh32i:

 	mov	r7, a
	acall	asc2hex
	jc	gh32c
	xch	a, r7
	lcall	cout
	mov	a, r7
	swap	a
	mov	r7, a
	mov	r0, #4

gh32j:

	mov	a, r7
	rlc	a
	mov	r7, a
	mov	a, r3
	rlc	a
	mov	r3, a
	mov	a, r4
	rlc	a
	mov	r4, a
	mov	a, r5
	rlc	a
	mov	r5, a
	mov	a, r6
	rlc	a
	mov	r6, a
	djnz	r0, gh32j
	djnz	r2, gh32c
	clr	c
	ret

;carry set if invalid input

asc2hex:

	clr     c
       	add     a, #208
        jnc     hex_not
	add     a, #246
        jc      hex_maybe
	add     a, #10
        clr     c
        ret

hex_maybe:

	add     a, #249
        jnc     hex_not
        add     a, #250
        jc      hex_not
        add     a, #16
        clr     c
        ret

hex_not:

	setb    c
        ret

;print a hexdump of the data in the 512 byte buffer
hexdump:

	lcall	newline
	mov	r2, #32		;print 32 lines
	mov	dptr, #buffer

hd1:

	mov	a, dph
	mov	r5, a
	clr	c
	subb	a, #buffer >> 8
	lcall	phex		;print address, starting at zero
	mov	a, dpl
	mov	r4, a
	lcall	phex
	mov	a, #':'
	lcall	cout
	acall	space
	mov	r3, #16		;print 16 hex bytes per line

hd2:

	movx	a, @dptr
	inc	dptr
	lcall	phex		;print each byte in hex
	acall	space
	djnz	r3, hd2
	acall	space
	acall	space
	acall	space
	mov	dpl, r4
	mov	dph, r5
	mov	r3, #16		;print 16 ascii bytes per line

hd3:
	
	movx	a, @dptr
	inc	dptr
	anl	a, #01111111b	;only 7 bits for ascii
	cjne	a, #127, hd3b
	clr	a		;avoid 127/255 (delete/rubout) char

hd3b:

	add	a, #224
	jc	hd3c
	clr	a		;avoid control characters

hd3c:

	add	a, #32
	lcall	cout
	djnz	r3, hd3
	lcall	newline
	djnz	r2, hd1
	lcall	newline
	ret

space:

	mov	a, #' '
	ljmp	cout

;------------------------------------------------------------------------
;HP1000 disk controller statup code begins, initialize the interface regs

dc_start:

;initialize the hp interface hardware

	clr	a			;clear accumulator
	mov	p1, a			;clear port1 signals
	mov	p1_buf, a		;update iram port1 variable
	mov	dptr, #hp_config		;get config address for hp interface
	mov	a, #10001001b		;get hp 8255 write mode config byte 
	movx	@dptr, a		;write to hp interface 8255

;clear the hp interface data registers
	mov	dptr, #hp_porta		;get hp-8255 port a address
	clr	a			;clear accumulator			
	movx	@dptr, a		;write 0's to port a
	mov	dptr, #hp_portb		;get hp-8255 port b address
	movx	@dptr, a		;write 0's to port b
	lcall 	ck_bus			;clock bus to hp data registers

;clear ide error code buffer

	clr	a			;clear accumulator
	mov	ide_error, a		;save in ide error code buffer

;reset disk controller

	mov	blk, #0x00		;zero blk+0
	mov	blk+1, #0x00		;zero blk+1
	mov	blk+2, #0x00		;zero blk+2
	mov	blk+3, #0x00		;zero blk+3
	lcall 	blk_2_lba		;update ATA address value

	clr	acc			;clear accumulator
	mov	dc_len, a		;zero transfer lenght counter
	mov	ide_error, a		;clear ide error code buffer

;check the front panel mode switch

	mov	dptr, #hp_portc		;get hp-8255 port c address
	movx	a, @dptr		;read port c into accumulator
	anl	a, #00000001b		;mask out mode switch bit
	jnz	hp_disk			;is switch up?	
	ljmp	debug			;no, start command line debugger


;-------------------------------------------------------------------
; HP1000 disk controller code begins here
;enter the disk controller command state, clear operate and set
;the command bits

hp_disk:
	mov	a, p1_buf		;get port1 state
	orl	a, #00000001b		;set command led bit
	anl	a, #11111101b		;clear operate led bit
	mov	p1_buf, a		;save new buffer value
	mov	p1, a			;send new value to port1

;send a debugging message to the 8052 console

;	mov	dptr, #d_msg1		;get command string address
;	lcall	pstr			;send it to the debug console


;****************************************************************

; code added to send IDE controller status word to the HP 
; interface while waiting to receive a new command.


;generate the disk controller status word

	mov	a, p1_buf			;get port1 buffer value
	anl	a, #0x10			;mask out error bit only
	rl	a
	rl	a
	rl	a				;move error bit to msb
	mov	hp_msb, a			;save result to I/O buffer
	mov	a, ide_error			;get ide error value
	mov	hp_lsb, a			;save error value to I/O buffer

;clear d_data bit so 8255 can drive bus0-15

	mov	a, p1_buf			;get port1 state
	anl	a, #10111111b			;clear d_data bit
	mov	p1_buf, a			;save new port1 state
	mov	p1, a				;send new value to port1

;configure 8255 to drive bus0-15

	mov	dptr, #hp_config		;get hp 8255 config address
	mov	a, #10001001b			;get output config byte
	movx	@dptr, a			;write it to the config port

;send controller status to hp interface

	mov	dptr, #hp_porta			;get hp 8255 porta address
	mov	a, hp_lsb			;get ide status byte
	movx	@dptr, a			;send it to bus0-7
	mov	dptr, #hp_portb			;get hp 8255 portb address
	mov	a, hp_msb			;get disk controller status value
	movx	@dptr, a			;send it to bus8-15
	lcall ck_bus				;clock bus into hp data registers

;****************************************************************


;get a command word from the hp interface

	lcall hp_2_ide			;get command word from hp

;----------------------------------------------------------------
;enter the disk controller operate state, clear command and set
;the operate bits

	mov	a, p1_buf		;get port1 value
	anl	a, #11111110b		;clear command led bit
	orl	a, #00000010b		;set operate led bit
	mov	p1_buf, a		;save new port1 value
	mov	p1, a			;send new value to port1

;process disk command / parameter word

	mov	a, hp_lsb		;get hp lsb, the parameter byte 
	mov	dc_para, a		;save hp parameter value
	mov	a, hp_msb		;get hp msb, the command byte
	mov	dc_cmd, a		;save hp command value

;------------------------------------------------------------------
;Display HP disk controller debugging command and parameter bytes

;	mov	dptr, #d_msg2		;get operate mode string address
;	lcall	pstr			;send it to the debug console
;	mov	a, dc_cmd		;get command byte
;	lcall	phex			;print it to the debug console
;	mov	dptr, #d_msg3		;get parameter string address
;	lcall pstr			;send it to the debug console
;	mov	a, dc_para		;get parameter byte
;	lcall phex			;print it to the debug console
;	lcall newline			;end of debug display line

;------------------------------------------------------------------
;disk command execution begins

dx0:
	mov	a, dc_cmd		;get command value
	cjne	a, #0x80, dx1		;is it 0x80, Reset Disk Controller?
	ljmp	rdc			;yes, jump to rdc code

;-----------------------------------------------------------
;its not reset, so check for any pre-existing error state
dx1:

	mov	a, p1_buf		;get port1 value
	anl	a, #0x10		;mask error bit only
	jnz	dx_err			;got existing error state?

;no error set, proceed to process commands other than reset
	mov	a, dc_cmd		;get command value
	cjne	a, #0x82, dx2		;is it 0x82, Set Block 1?
	ljmp	sb1			;yes, jump to sb1 code
	
dx_err:
	ljmp	dx_done			;error is set, accept only RDC!

dx2:
	cjne	a, #0x84, dx3		;is it 0x84, Set Block 2?
	ljmp	sb2			;yes, jump to sb2 code

dx3:
	cjne	a, #0x86, dx4		;is it 0x86, Set Block 3?
	ljmp	sb3			;yes, jump to sb3 code

dx4:
	cjne	a, #0x88, dx5		;is it 0x88, Set Block 4?
	ljmp	sb4			;yes, jump to sb4 code

dx5:
	cjne	a, #0x90, dx6		;is it 0x90, ReaD Blocks?
	ljmp	rdb			;yes, jump to rdb code

dx6:
	cjne	a, #0x92, dx7		;is it 0x92, WRite Blocks?
	ljmp	wrb			;yes, jump to wrb code

dx7:

;--------------------------------------------------------------
; Add new disk controller command functions here



;--------------------------------------------------------------
	
dx_error:

;unrecognised command byte detected!
;set disk controller error status bit

;send debugging status message

;	mov	dptr, #d_msg4		;get error string address
;	lcall	pstr			;send it to the debug console

;set error state
	mov	a, p1_buf		;get port1 buffer status
	orl	a, #00010000b		;set error bit
	mov	p1_buf, a		;save new buffer value
	mov	p1, a			;send new value to port1

;and complete HP command protocol

dx_done:
;send controller status word to hp interface, an all zero
;word signals that the ide controller is ready and awaiting
;a valid command from the HP.

	lcall	up_stat			;update controller status
	ljmp	hp_disk			;and get another command

;------------------------------------------------------------------------
;disk controller command execution functions begin...
;------------------------------------------------------------------------

;RDC - Reset Disk Controller
;reset block, lba, lenght, and error values to zero, clear error state
rdc:

	mov	blk, #0x00		;zero blk+0
	mov	blk+1, #0x00		;zero blk+1
	mov	blk+2, #0x00		;zero blk+2
	mov	blk+3, #0x00		;zero blk+3
	lcall blk_2_lba			;update ATA address value

	clr	acc			;clear accumulator
	mov	dc_len, a		;zero transfer lenght counter
	mov	ide_error, a		;clear ide error code buffer

;clear error state flag and led
	mov	a, p1_buf		;get port1 state
	anl	a, #11101111b		;clear error led
	mov	p1_buf, a		;save new port1 value
	mov	p1, a			;send new value to port1
	ljmp	dx_done			;end of RDC function

;-----------------------------------------------------------------------
;SB1 - Set Block 1
;sets low 8 bits of 1K word block address
sb1:
	mov	a, dc_para		;get disk parameter byte
	mov	blk, a			;save 8 bits of 1K word block address
;	lcall 	p_blk			;display updated HP block address
	lcall	blk_2_lba		;update the ata lba value
;	lcall 	p_lba			;display updated lba address
	ljmp	dx_done			;end of SB1 function

;-----------------------------------------------------------------------
;Set Block 2
;sets next to least signifigent byte of 1K word block address

sb2:
	mov	a, dc_para		;get disk parameter byte
	mov	blk+1, a		;save 8 bits of 1K word block address
;	lcall 	p_blk			;display updated HP block address
	lcall	blk_2_lba		;update the ata lba value
;	lcall	p_lba			;display updated lba address
	ljmp	dx_done			;end of SB2 function

;-----------------------------------------------------------------------
;Set Block 3
;sets next to most signifigent byte of 1K word block address

sb3:
	mov	a, dc_para		;get disk parameter byte
	mov	blk+2, a		;save 8 bits of 1K word block address
;	lcall	p_blk			;display updated HP block address
	lcall	blk_2_lba		;update the ata lba value
;	lcall	p_lba			;display updated lba address
	ljmp	dx_done			;end of SB3 function

;-----------------------------------------------------------------------
;Set Block 4
;sets top 2 bits of 1K word block address

sb4:
	mov	a, dc_para		;get disk parameter byte

;mask for valid 26-bit 1K word block address

	anl	a, #00000011b		;mask off all invalid bits
	mov	blk+3, a		;save 2 bits of 1K word block address
;	lcall 	p_blk			;display updated HP block address
	lcall	blk_2_lba		;update the ata lba value
;	lcall	p_lba			;display updated lba address
	ljmp	dx_done			;end of SB4 function

;-----------------------------------------------------------------------
; RDB - ReaD Blocks.  RDB with lenght = 0 will read one block
; from disk into buffer.  RDB with lenght > 0 will read and
; transfer (lenght) 1K word blocks to the HP interface.

rdb:	

;echo command word back to host, d_data is still set!

	lcall	ck_bus			;clock bus data into hp data registers

;complete the hp to ide command transfer protocol

	lcall	ck_flg			;clock flag

;get transfer lenght value from parameter byte

	mov	a, dc_para		;get transfer lenght
	
;mask to valid lenght and save transfer lenght value

	anl	a, #00011111b		;mask off high 3 bits
	mov	dc_len, a		;save new lenght value

rb_loop:
	
;begin / continue read blocks operation

;	lcall	p_lba			;display the current lba address
	lcall	rd_blk			;read 1 block from disk to buffer
	mov	a, dc_len		;get current transfer lenght
	jz	rb_done			;ready to transfer blocks?

;decrement transfer lenght
	mov	a, dc_len		;get transfer lenght
	dec	a			;decrement value
	mov	dc_len, a		;save decremented value

;send 1K word of data to the HP

	lcall tx_blk			;send buffer to HP

;are we done with block transfers to HP?

	mov	a, dc_len		;get transfer lenght
	jz	rb_done			;done with block transfers

;no, more blocks remain to read and transfer
	
	ljmp rb_loop			;proceed with next block

rb_done:
	ljmp hp_disk			;done with RDB function

;----------------------------------------------------------
; WRB - Write Blocks.  WRB with lenght = 0 will write one block
; from buffer onto  disk.  WRB with lenght > 0 will transfer
; and write (lenght) 1K word blocks from the HP interface to
; disk.
	
wrb:

;echo command word back to host, d_data is still set!

	lcall	ck_bus			;clock bus data into hp data registers

; complete the hp to ide command transfer protocol

	lcall	ck_flg			;clock flag

;get transfer lenght value from parameter byte

	mov	a, dc_para		;get transfer lenght
	
;mask for invalid lenght

	anl	a, #00011111b		;mask off high 3 bits

;save new transfer lenght value

	mov	dc_len, a		;save new lenght value

;is the transfer lenght equal to zero?

	mov	a, dc_len		;get current transfer lenght
	jz	xb_done			;any blocks to transfer?

wb_loop:

;get 1K word of data from the host into the buffer

	lcall rx_blk			;get 1024 words from HP into buffer

xb_done:

;write the current buffer contents to disk

;	lcall	p_lba			;display the current lba address
	lcall wr_blk			;write current buffer to disk

;is the transfer lenght now equal to zero?

	mov	a, dc_len		;get current transfer lenght
	jz	wb_done			;any more blocks to transfer?

	mov	a, dc_len		;get current transfer lenght
	dec	acc			;lenth = lenght - 1
	mov	dc_len, a		;save new transfer lenght
	jnz	wb_loop			;more blocks to transfer?

wb_done:

	ljmp hp_disk			;no, WRB function completed
	
;------------------------------------------------------------

up_stat:

;generate the disk controller status word

	mov	a, p1_buf			;get port1 buffer value
	anl	a, #0x10			;mask out error bit only
	rl	a
	rl	a
	rl	a				;move error bit to msb
	mov	hp_msb, a			;save result to I/O buffer
	mov	a, ide_error			;get ide error value
	mov	hp_lsb, a			;save error value to I/O buffer

;clear d_data bit so 8255 can drive bus0-15

	mov	a, p1_buf			;get port1 state
	anl	a, #10111111b			;clear d_data bit
	mov	p1_buf, a			;save new port1 state
	mov	p1, a				;send new value to port1

;configure 8255 to drive bus0-15

	mov	dptr, #hp_config		;get hp 8255 config address
	mov	a, #10001001b			;get output config byte
	movx	@dptr, a			;write it to the config port

;send controller status to hp interface

	mov	dptr, #hp_porta			;get hp 8255 porta address
	mov	a, hp_lsb			;get ide status byte
	movx	@dptr, a			;send it to bus0-7
	mov	dptr, #hp_portb			;get hp 8255 portb address
	mov	a, hp_msb			;get disk controller status value
	movx	@dptr, a			;send it to bus8-15
	lcall ck_bus				;clock bus into hp data registers
	lcall	ck_flg				;complete command transfer
	ret					;end of up_stat

;--------------------------------------------------------------------

blk_2_lba:

	mov	a, blk				;get low byte of block address
	rl	a				;rotate left one bit
	rl	a				;rotate left another bit
	anl	a, #11111100b			;mask off low 2 bits
	mov	lba, a				;save in lba+0

;ok so far, lba+0 is done, now claculate lba+1

	mov	a, blk				;get low byte again
	anl	a, #11000000b			;keep high 2 bits
	rl	a				;rotate left one bit
	rl	a				;rotate left another bit
	mov	tmp, a				;save in tmp+0
	mov	a, blk+1			;get next higher byte
	rl	a				;rotate left one bit
	rl	a				;rotate left another bit
	anl	a, #11111100b			;mask off low 2 bits
	orl	a, tmp				;or in tmp+0 value
	mov	lba+1, a			;save in lba+1

;ok, lba+1 is now done, now calculate lba+2

	mov	a, blk+1			;get next higher byte again
	anl	a, #11000000b			;keep high 2 bits
	rl	a				;rotate left one bit
	rl	a				;roate left another bit
	mov	tmp+1, a			;save in tmp+1
	mov	a, blk+2			;get even higher byte
	rl	a				;rotate left one bit
	rl	a				;rotate left another bit
	anl	a, #11111100b			;mask off low 2 bits
	orl	a, tmp+1			;or in tmp+1 value
	mov	lba+2, a			;save in lba+2

;then calculate the value for lba+3

	mov	a, blk+2			;get even higher byte again
	anl	a, #11000000b			;keep high 2 bits
	rl	a				;rotate left one bit
	rl	a				;rotate left another bit
	mov	tmp+2, a			;save in tmp+2
	mov	a, blk+3			;get top bits of 1K block address
	rl	a				;rotate left one bit
	rl	a				;rotate left another bit
	anl	a, #11111100b			;mask off low 2 bits
	orl	a, tmp+2			;or-in tmp+2 value
	mov	lba+3, a			;save in lba+3
	ret					;end of blk_2_lba
			

;-------------------------------------------------------------
; 1K word logical block operations

rd_blk:

;read 1024 16-bit words from drive into buffer

	lcall rd_sec_1				;read first 512 byte lba
	lcall inc_lba				;increment lba pointer
	lcall rd_sec_2				;read second 512 byte lba
	lcall inc_lba				;increment lba pointer
	lcall rd_sec_3				;read third 512 byte lba
	lcall inc_lba				;increment lba pointer
	lcall rd_sec_4				;read fourth 512 byte lba
	lcall inc_lba				;increment lba pointer
	ret					;end of read block

wr_blk:

;write 1024 16-bit words from buffer to drive

	lcall wr_sec_1				;write first 512 byte lba
	lcall inc_lba				;increment lba pointer
	lcall wr_sec_2				;write second 512 byte lba
	lcall inc_lba				;increment lba pointer
	lcall wr_sec_3				;write third 512 byte lba
	lcall inc_lba				;increment lba pointer
	lcall wr_sec_4				;write fourth 512 byte lba
	lcall inc_lba				;increment lba pointer
	ret					;end of write block

;-------------------------------------------------------------------

tx_blk:

;set the block output mode

	mov	a, p1_buf			;get the port1 value
	orl	a, #00001000b			;set block output bit
	mov	p1_buf, a			;save the new port1 value
	mov	p1, a				;send new value to port1

;send 1024 16-bit words to HP with handshaking.

	mov	dptr, #buff_1		;point to start of disk buffer
	mov	r5, #0			;preset loop counter

tx_c_loop:

;transfer word from the buffer to the host 

	movx	a, @dptr			;read msb from buffer
	mov	hp_msb, a			;save to I/O buffer
	inc	dptr				;point to lsb
	movx	a, @dptr			;read lsb from buffer
	mov	hp_lsb, a			;save to I/O buffer
	inc	dptr				;point to msb of next word		
	lcall	ide_2_hp			;send the word to the HP

;transfer word from the buffer to the host 

	movx	a, @dptr			;read msb from buffer
	mov	hp_msb, a			;save to I/O buffer
	inc	dptr				;point to lsb
	movx	a, @dptr			;read lsb from buffer
	mov	hp_lsb, a			;save to I/O buffer
	inc	dptr				;point to msb of next word		
	lcall	ide_2_hp			;send the word to the HP

;transfer word from the buffer to the host 

	movx	a, @dptr			;read msb from buffer
	mov	hp_msb, a			;save to I/O buffer
	inc	dptr				;point to lsb
	movx	a, @dptr			;read lsb from buffer
	mov	hp_lsb, a			;save to I/O buffer
	inc	dptr				;point to msb of next word		
	lcall	ide_2_hp			;send the word to the HP

;transfer word from the buffer to the host
 
	movx	a, @dptr			;read msb from buffer
	mov	hp_msb, a			;save to I/O buffer
	inc	dptr				;point to lsb
	movx	a, @dptr			;read lsb from buffer
	mov	hp_lsb, a			;save to I/O buffer
	inc	dptr				;point to msb of next word		
	lcall	ide_2_hp			;send the word to the HP

;update and test 1K word block counter (R5)

	djnz	r5, tx_c_loop			;repeat loop 256 times

;clear the block output mode and exit

	mov	a, p1_buf			;get the port1 value
	anl	a, #11110111b			;clear the block output bit
	mov	p1_buf, a			;save the new value
	mov	p1, a				;send it to port1
	ret					;end of 1K word block transmit

;-----------------------------------------------------------------

rx_blk:

;set the block input mode

	mov	a, p1_buf			;get the port1 value
	orl	a, #00000100b			;set the block input mode
	mov	p1_buf, a			;save new port1 value
	mov	p1, a				;send new value to port1

;receive 1024 16-bit words from HP with handshaking.

	mov	dptr, #buff_1			;point to start of buffer
	mov	r5, #0				;preset loop counter

rx_c_loop:

;transfer one word from the host into the disk data buffer

	lcall hp_2_ide				;get word from host
	lcall	ck_flg				;acknowledge transfer
	mov	a, hp_msb			;get msb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer
	mov	a, hp_lsb			;get lsb from I/O buffer
	movx	@dptr, a			;store into buffer
	inc dptr				;increment buffer pointer

;transfer one word from the host into the buffer

	lcall hp_2_ide				;get word from host
	lcall	ck_flg				;acknowledge transfer
	mov	a, hp_msb			;get msb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer
	mov	a, hp_lsb			;get lsb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer

;transfer one word from the host into the buffer

	lcall hp_2_ide				;get word from host
	lcall ck_flg				;acknowlege transfer
	mov	a, hp_msb			;get msb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer
	mov	a, hp_lsb			;get lsb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer

;transfer one word from the host into the buffer

	lcall hp_2_ide				;get word from host
	lcall ck_flg				;acknowlege transfer
	mov	a, hp_msb			;get msb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer
	mov	a, hp_lsb			;get lsb from I/O buffer
	movx	@dptr, a			;store into disk buffer
	inc dptr				;increment buffer pointer

;update and test 1K word block counter (R5)

	djnz	r5, rx_c_loop			;repeat loop 256 times

;clear block input mode and exit

	mov	a, p1_buf			;get port1 buffer value
	anl	a, #11111011b			;clear block input bit
	mov	p1_buf, a			;save new port1 value
	mov	p1, a				;send new value to port1
	ret					;end of 1K word block receive


;---------------------------------------------------------------
; HP hardware interface functions

ck_bus:	

; clock bus0-15 into hp data registers

	mov	a, p1_buf			;get port1 state
	orl	a, #00100000b			;and latch data bit
	mov	p1, a				;send to port1
	mov	a, p1_buf			;get port1 state
	mov	p1, a				;restore original port1 state
	ret					;return

;----------------------------------------------------------------

ck_flg:

; clock the flag flip flop on hp interface board
	mov	a, p1_buf			;get port1 state
	orl	a, #10000000b			;or-in hp flag bit
	mov	p1, a				;send to port1
	mov	a, p1_buf			;get port1 state
	mov	p1, a				;restore original port1 state
	ret					;return

;-----------------------------------------------------------------

w_cmd:

; wait for command signal from hp interface board
	mov	dptr, #hp_portc			;get hp 8255 port c address

w_cmd1:						;keep checking for hp_cmd

	movx	a, @dptr			;read hp 8255 port c
	anl	a, #00000010b			;mask hp-command signal
	jz	w_cmd1				;is hp_cmd asserted?
	ret					;yes, we are done

;-----------------------------------------------------------------

hp_2_ide:

; get a word from the hp interface with handshaking
; the 16-bit word will be returned in hp_msb and 
; hp_lsb buffers.  does not set flag after the
; data is read, so the HP will keep waiting for
; the transfer to complete in skip-on-flag loop.
; d_data is left SET on exit, 8255 in input mode!

; save contents of dptr

	push	dpl				;save low byte of dptr
	push	dph				;save high byte of dptr

; configure 8255 to read bus0-15

	mov	dptr, #hp_config		;get hp 8255 config address
	mov	a, #10011011b			;get input config byte
	movx	@dptr, a			;write it to the config port

; set d_data bit so hp output data drives bus0-15

	mov	a, p1_buf			;get port1 state
	orl	a, #01000000b			;or-in the d_data bit
	mov	p1_buf, a			;save new port1 value
	mov	p1, a				;send new value to port1

; wait for device command signal from hp interface

	lcall w_cmd				;wait for hp device command signal

; read word from hp interface

	mov	dptr, #hp_portb			;get hp 8255 port b address
	movx	a, @dptr			;read bus8-15 into accumulator
	mov	hp_msb, a			;save msb to buffer
	mov	dptr, #hp_porta			;get hp 8255 port a address
	movx	a, @dptr			;read bus0-7 into accumulator
	mov	hp_lsb, a			;save lsb to buffer

; restore contents of dptr

	pop	dph				;restore high byte of dptr
	pop	dpl				;restore low byte of dptr
	ret					;end of hp_2_ide

ide_2_hp:

; send a word to the hp interface with handshaking
; the 16-bit word to be sent must be in the hp_msb
; and hp_lsb data buffers.

; save contents of dptr

	push	dpl				;save low byte of dptr
	push	dph				;save high byte of dptr

; clear d_data bit so 8255 can drive bus0-15

	mov	a, p1_buf			;get port1 state
	anl	a, #10111111b			;clear d_data bit
	mov	p1_buf, a			;save new port1 state
	mov	p1, a				;send new value to port1

; configure 8255 to drive bus0-15

	mov	dptr, #hp_config		;get hp 8255 config address
	mov	a, #10001001b			;get output config byte
	movx	@dptr, a			;write it to the config port

; wait for device command signal from hp interface

	lcall w_cmd				;wait for cmd function

; send the word to bus0-15
	mov	dptr, #hp_porta			;get hp 8255 port a address
	mov	a, hp_lsb			;get lsb of 16-bit word
	movx	@dptr, a			;send it to hp 8255 port a
	mov	dptr, #hp_portb			;get hp 8255 port b address
	mov	a, hp_msb			;get msb of 16-bit word
	movx	@dptr, a			;send it to hp 8255 port b

; clock the data into the hp data registers

	lcall ck_bus				;pulse the l_data signal

; complete the transfer by clocking flag

	lcall ck_flg				;clock flag to clear hp_cmd

; restore contents of dptr

	pop	dph				;restore high byte of dptr
	pop	dpl				;restore low byte of dptr
	ret					;end of ide_2_hp
	
;--------------------------------------------------------------
; HP1000 interface hardware test functions for Debug Mode.
;--------------------------------------------------------------

int_tests:

;display the interface test menu and get user response

	mov	dptr, #msg_2			;get interface test menu string
	lcall	pstr				;send it to the console
	lcall	cin				;get users selection
	lcall	upper				;convert to upper case

; test the users selection, was is (R)ead?

	cjne a, #'R', int2			;was is (R)ead?

; yes,  configure 8255 to read bus0-15

	mov	dptr, #hp_config		;get hp 8255 config address
	mov	a, #10011011b			;get input config byte
	movx	@dptr, a			;write it to the config port

; set d_data bit so hp output data drives bus0-15

	mov	a, p1_buf			;get port1 state
	orl	a, #01000000b			;or-in the d_data bit
	mov	p1_buf, a			;save new port1 value
	mov	p1, a				;send new value to port1

; read word from hp interface

	mov	dptr, #hp_portb			;get hp 8255 port b address
	movx	a, @dptr			;read bus8-15 into accumulator
	push	acc				;push msb to stack
	mov	dptr, #hp_porta			;get hp 8255 port a address
	movx	a, @dptr			;read bus0-7 into accumulator
	push	acc				;push lsb to stack

; clear d_data bit so 8255 data may drive bus0-15

	mov	a, p1_buf			;get port1 state
	anl	a, #10111111b			;clear the d_data bit
	mov	p1_buf, a			;save new port1 value
	mov	p1, a				;send new value to port1


; display hp data word read from hp

	mov	dptr, #msg_rv			;get read value message address
	lcall pstr				;send it to the console
	pop 	dpl				;pop lsb from stack
	pop 	dph				;pop msb from stack
	lcall phex16				;print 16 bit hex value
	lcall newline				;cr,lf sequence
	ljmp	int_tests			;restart interface menu


int2:
	cjne a, #'W', int3			;no, was is (W)rite?

; yes, clear d_data bit so 8255 can drive bus0-15

	mov	a, p1_buf			;get port1 state
	anl	a, #10111111b			;clear d_data bit
	mov	p1_buf, a			;save new port1 state
	mov	p1, a				;send new value to port1

; configure 8255 to drive bus0-15

	mov	dptr, #hp_config		;get hp 8255 config address
	mov	a, #10001001b			;get output config byte
	movx	@dptr, a			;write it to the config port

int2b:

; get 16-bit value to send

	mov	dptr, #msg_wv			;get write prompt address
	lcall pstr				;send it to the console
	lcall ghex16				;get data to send
	jc	int2a				;got valid input?
	lcall newline
	
; write data to interface bus

	push 	dph				;save msb
	push 	dpl				;save lsb
	mov	dptr, #hp_porta			;get hp 8255 porta address
	pop	acc				;get lsb data
	movx	@dptr, a			;send lsb to bus0-7
	mov	dptr, #hp_portb			;get hp 8255 portb address
	pop	acc				;get msb data
	movx	@dptr, a			;send msb to bus8-15

; clock data into hp data registers

	lcall ck_bus				;clock data registers
	ljmp	int_tests			;restart interface menu

; handle invalid hex input

int2a:

	lcall newline				;get off bad input line
	ljmp int2b				;try again for valid input

int3:

	cjne a, #'S', int4
	lcall ck_flg				;clock the interface flag
	ljmp	int_tests			;restart interface menu

int4:

	cjne a, #'Q', int5
	ljmp	main_loop			;restart main menu

int5:

	ljmp int_tests				;restart interface menu	

msg_2:	.db	"(R)ead (W)rite (S)et flag (Q)uit",13,10,0
msg_rv:	.db	"HP Data = ",0
msg_wv:	.db	"Disk Data = ",0

;-----------------------------------------------------------
;increment the 4-byte logical block address value

inc_lba:					; increment LBA pointer

	mov	r3, lba+0			; get low 8 bits of lba
	mov	r4, lba+1			; get next 8 bits
	mov	r5, lba+2			; get another 8 bits
	mov	r6, lba+3			; get top bits of lba

	mov	a, r3				; get byte
	inc	a				; add one
	mov	r3, a				; save low lba byte
	jz	nxt_1				; continue if result is zero
	sjmp	nxt_lba_1			; if not, we are done

nxt_1:

	mov	a, r4				; get byte
	inc	a				; add one
	mov	r4, a				; save next lba byte
	jz	nxt_2				; continue if result is zero
	sjmp	nxt_lba_1			; if not, we are done

nxt_2:

	mov	a, r5				; get byte
	inc	a				; add one
	mov	r5, a				; save next lba byte
	jz	nxt_3				; continue if result is zero
	sjmp	nxt_lba_1			; if not, we are done

nxt_3:

	mov	a, r6				; get byte
	inc	a				; add one
	mov	r6, a				; save high lba byte	

nxt_lba_1:					; save and check for end of disk

	mov	lba+0, r3			; save low 8 bits of lba
	mov	lba+1, r4			; save next byte of lba
	mov	lba+2, r5
	mov	lba+3, r6			; save hi updated lba value
	ret					; return from inc_lba

;------------------------------------------------------------
;break:
;	ljmp	break				;hang here for debugging!
;------------------------------------------------------------


d_msg1:	.db	"Command mode.",10,13,10,13,0
d_msg2:	.db	"Operate mode, command = ",0
d_msg3:	.db	" parameter = ",0
d_msg4:	.db	"Unrecognised command.",10,13,0
d_msg5:	.db	"ATA Logical Block Address = 0x",0
d_msg6:	.db	"HP Disk Block Address = 0x",0

;--------------------------------------
;Multi-sector (block) reads and writes!
;--------------------------------------

;------------------------------------------------------------
;read the first sector of a 1K word block, using the current
;lba address values.
;Return, acc is zero on success, non-zero for an error

rd_sec_1:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_read
	lcall	ide_wr				;ask the drive to read it
	lcall	ide_drq				;wait until it's got the data
	jb	acc.0, get_err_1
	mov	dptr, #buff_1			;point to first 512 bytes of buffer
	lcall	read_data			;grab the data
	clr	a
	mov	ide_error, a			;save success state
	ret

get_err_1:

	mov	a, #ide_err
	lcall	ide_rd
	mov	a, r2
	jz	gerr1
	mov	ide_error, a			;save ide error code
	lcall	set_error			;set error bit
	ret

gerr1:	

	mov	a, #255
	mov	ide_error, a			;save ide error code
	lcall	set_error			;set error
	ret


;write the first sector of a 1K word block using the current
;lba address values.
;Return, acc is zero on success, non-zero for an error

wr_sec_1:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_write
	lcall	ide_wr				;tell drive to write a sector
	lcall	ide_drq				;wait unit it wants the data
	jb	acc.0, get_err_1
	mov	dptr, #buff_1			;point to first 512 bytes of buffer
	lcall	write_data			;give the data to the drive
	lcall	ide_busy			;wait until the write is complete
	jb	acc.0, get_err_1
	clr	a
	mov	ide_error, a			;save success state
	ret

;--------------------------------------------------------------------

;read the second sector of a 1K word block using the current lba
;address values.
;Return, acc is zero on success, non-zero for an error

rd_sec_2:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_read
	lcall	ide_wr				;ask the drive to read it
	lcall	ide_drq				;wait until it's got the data
	jb	acc.0, get_err_2
	mov	dptr, #buff_2			;point to second 512 bytes of buffer
	lcall	read_data			;grab the data
	clr	a
	mov	ide_error, a			;save success state
	ret

get_err_2:

	mov	a, #ide_err
	lcall	ide_rd
	mov	a, r2
	jz	gerr2
	mov	ide_error, a			;save ide error code
	lcall set_error				;set error bit
	ret

gerr2:
	
	mov	a, #255
	mov	ide_error, a			;save ide error code
	lcall set_error				;set error bit
	ret


;write the second sector of a 1K word block using the current
;lba address values.
;Return, acc is zero on success, non-zero for an error

wr_sec_2:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_write
	lcall	ide_wr				;tell drive to write a sector
	lcall	ide_drq				;wait unit it wants the data
	jb	acc.0, get_err_2
	mov	dptr, #buff_2			;point to second 512 bytes of buffer
	lcall	write_data			;give the data to the drive
	lcall	ide_busy			;wait until the write is complete
	jb	acc.0, get_err_2
	clr	a
	mov	ide_error, a			;save success state
	ret

;--------------------------------------------------------------------
;read the third sector of a 1K word block using the current lba
;address values.
;Return, acc is zero on success, non-zero for an error

rd_sec_3:
	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_read
	lcall	ide_wr				;ask the drive to read it
	lcall	ide_drq				;wait until it's got the data
	jb	acc.0, get_err_3
	mov	dptr, #buff_3			;point to third 512 bytes of buffer
	lcall	read_data			;grab the data
	clr	a
	mov	ide_error, a			;save success state
	ret

get_err_3:

	mov	a, #ide_err
	lcall	ide_rd
	mov	a, r2
	jz	gerr3
	mov	ide_error, a			;save ide error code
	lcall set_error				;set error bit
	ret

gerr3:

	mov	a, #255
	mov	ide_error, a			;save ide error code
	lcall set_error				;set error bit
	ret


;write the third sector of a 1K word block using the current lba
;address values.
;Return, acc is zero on success, non-zero for an error

wr_sec_3:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_write
	lcall	ide_wr				;tell drive to write a sector
	lcall	ide_drq				;wait unit it wants the data
	jb	acc.0, get_err_3
	mov	dptr, #buff_3			;point to third 512 bytes of buffer
	lcall	write_data			;give the data to the drive
	lcall	ide_busy			;wait until the write is complete
	jb	acc.0, get_err_4
	clr	a
	mov	ide_error, a			;save success state
	ret

;--------------------------------------------------------------------
;read the fourth sector of a 1K word block using the current lba
;address values.
;Return, acc is zero on success, non-zero for an error

rd_sec_4:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_read
	lcall	ide_wr				;ask the drive to read it
	lcall	ide_drq				;wait until it's got the data
	jb	acc.0, get_err_4
	mov	dptr, #buff_4			;point to fourth 512 bytes of buffer
	lcall	read_data			;grab the data
	clr	a
	mov	ide_error, a			;save success state
	ret

get_err_4:
	mov	a, #ide_err
	lcall	ide_rd
	mov	a, r2
	jz	gerr4
	mov	ide_error, a			;save ide error code
	lcall set_error				;set error bit
	ret
gerr4:
	mov	a, #255
	mov	ide_error, a			;save ide error code
	lcall set_error				;set error bit
	ret


;write the fourth sector of a 1K word block using the current lba
;address values.
;Return, acc is zero on success, non-zero for an error

wr_sec_4:

	lcall	ide_busy			;make sure drive is ready
	lcall	wr_lba				;tell it which sector we want
	mov	a, #ide_command
	mov	r2, #ide_cmd_write
	lcall	ide_wr				;tell drive to write a sector
	lcall	ide_drq				;wait unit it wants the data
	jb	acc.0, get_err_4
	mov	dptr, #buff_4			;point to fourth 512 bytes of buffer
	lcall	write_data			;give the data to the drive
	lcall	ide_busy			;wait until the write is complete
	jb	acc.0, get_err_4
	clr	a
	mov	ide_error, a			;save success state
	ret

;------------------------------------------------------------------------

set_error:

	mov	a, p1_buf			;get port1 state
	orl	a, #00010000b			;set error bit
	mov	p1_buf, a			;save new port1 value
	mov	p1, a				;send new value to port1
	ret					;end of set_error

;------------------------------------------------------------------------

p_lba:

	push	dph				;save dph to stack
	push	dpl				;save dpl to stack

	mov	dptr, #d_msg5			;point to lba address message
	lcall pstr				;send it to the console

	mov	a, lba+3
	lcall	phex				;print lba msb
	mov	a, lba+2
	lcall	phex				;print next lsb
	mov	a, lba+1
	lcall	phex				;print next lsb
	mov	a, lba+0
	lcall	phex				;print the lsb
	lcall newline				;send cr, lf to console	

	pop	dpl				;restore dpl
	pop	dph				;restore dph
	ret					;end of p_lba



;--------------------------------------------------------------------------

p_blk:

	push	dph				;save dph to stack
	push	dpl				;save dpl to stack

	mov	dptr, #d_msg6			;point to block address message
	lcall pstr				;send it to the console

	mov	a, blk+3
	lcall	phex				;print lba msb
	mov	a, blk+2
	lcall	phex				;print next lsb
	mov	a, blk+1
	lcall	phex				;print next lsb
	mov	a, blk+0
	lcall	phex				;print the lsb
	lcall newline				;send cr, lf to console	

	pop	dpl				;restore dpl
	pop	dph				;restore dph
	ret					;end of p_lba



